#
# This file is part of the Ingram Micro CloudBlue Connect Python OpenAPI Client.
#
# Copyright (c) 2021 Ingram Micro. All Rights Reserved.
#
CONNECT_SPECS_URL = 'https://apispec.connect.cloudblue.com/connect-openapi30.yml'  # noqa
CONNECT_ENDPOINT_URL = 'https://api.connect.cloudblue.com/public/v1'  # noqa
